<?php

namespace SM\Service;

use SM\Factory\ContractModelFactory;
use SM\Service\FindService;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class ContractModelService
{
    protected $ContractModelFactory;
    protected $findService;
    protected $Environment;
    protected $newCode;
    protected $connection;

    public function __construct(EntityManager $entityManager, ContractModelFactory $ContractModelFactory, FindService $findService, Environment $Environment, NewCode $newCode)
    {
        $this->connection               = $entityManager->getConnection();
        $this->ContractModelFactory     = $ContractModelFactory;
        $this->findService              = $findService;
        $this->Environment              = $Environment;
        $this->newCode                  = $newCode;
    }

    public function getContractModel()
    {
        return $this->ContractModelFactory->getContractModel();
    }

    public function getTabEscopoFaturamento($params)
    {
        $params = array(
            "NRSEQMODCONTRA"   => $params['NRSEQMODCONTRA']
        );
        return $this->ContractModelFactory->getTabEscopoFaturamento($params);
    }

    public function getTabModeloDocumento($params)
    {
        $params = array(
            "NRSEQMODCONTRA"   => $params['NRSEQMODCONTRA']
        );
    
        $retorno = $this->ContractModelFactory->getTabModeloDocumento($params);
        foreach ($retorno['data'] as $key => $value) {
            $retorno['data'][$key]['IDSITUACAOCONTRATO'] = explode(",", $retorno['data'][$key]['IDSITUACAOCONTRATO']);
        }

        return $retorno;
    }

    public function getTabPerfisAcesso($params)
    {
        $params = array(
            "NRSEQMODCONTRA"   => $params['NRSEQMODCONTRA']
        );
        return $this->ContractModelFactory->getTabPerfisAcesso($params);
    }

    public function getTabSignatariosPadroes($params)
    {
        $params = array(
            "NRSEQMODCONTRA"   => $params['NRSEQMODCONTRA']
        );
        return $this->ContractModelFactory->getTabSignatariosPadroes($params);
    }

    public function saveModalAddContractModel($params)
    {
        $this->connection->beginTransaction();

        try {

            $nrorg = $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();
            $nrSeqModContra = $this->newCode->getCodeByTableName("FCT_MODELOCONTRATO", $nrorg, 1, 10, true);

            $this->ContractModelFactory->saveModalAddContractModel(array(
                "NMMODCONTRATO"              => $params["NMMODCONTRATO"],
                "IDATIVO"                    => $params["IDATIVO"],
                "CDMODCONTRATO"              => $params["CDMODCONTRATO"],
                "IDGERAPROPOSTA"             => $params["IDGERAPROPOSTA"],
                "CDCENTCUST"                 => $params["CDCENTCUST"],
                "IDGERACONTFOR"              => $params["IDGERACONTFOR"],
                "NRDIASVIGCONT"              => $params["NRDIASVIGCONT"],
                "IDRENOVACAOAUTOMATICA"      => $params["IDRENOVACAOAUTOMATICA"],
                "NRDIASPRORGCONT"            => $params["NRDIASPRORGCONT"],
                "IDBLOQALTDTINICONTRENEG"    => $params["IDBLOQALTDTINICONTRENEG"],
                "IDREAJUSTEAUTOMATICO"       => $params["IDREAJUSTEAUTOMATICO"],
                "NRDIASREAJAUTOMATICO"       => $params["NRDIASREAJAUTOMATICO"],
                "IDINTEGRACAOFM"             => $params["IDINTEGRACAOFM"],
                "IDINTEGRALOCAISFM"          => $params["IDINTEGRALOCAISFM"],
                "IDINTEGRAMEDICAOFM"         => $params["IDINTEGRAMEDICAOFM"],
                "IDASSELETRDIGITAL"          => $params["IDASSELETRDIGITAL"],
                "NRTIPODOCCERTSIGN"          => $params["NRTIPODOCCERTSIGN"],
                "IDENVIADOCOPERRESPCONCASS"  => $params["IDENVIADOCOPERRESPCONCASS"],
                "NRORG"                      => $nrorg,
                "CDOPERINCLUSAO"             => $operador,
                "NRSEQMODCONTRA"             => $nrSeqModContra,
                "IDINTEGRACAOTECFOOD"        => $params["IDENVIADOCOPERRESPCONCASS"],
                "IDTIPOSERVINTTECFOOD"       => $params["IDTIPOSERVINTTECFOOD"],
                "IDINTEGRACAOHCM"            => $params["IDINTEGRACAOHCM"],
                "TPESTRUTINTEGRAHCM"         => $params["TPESTRUTINTEGRAHCM"],
                "IDINTEGRAESCALAHCM"         => $params["IDINTEGRAESCALAHCM"],
                "IDAGRUPAESCALAHCM"          => $params["IDAGRUPAESCALAHCM"],
            ));

            $this->insereEditaServicosExecHCM($nrSeqModContra, $params["CDSERVORCAEXECHCM"], $nrorg, $operador);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteContractModel($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->findService->formatarArray($parametros["NRS"]);
            $this->ContractModelFactory->deleteContractModel($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteContractModelTabEscopoFaturamento($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->findService->formatarArray($parametros["NRS"]);
            $this->ContractModelFactory->deleteContractModelTabEscopoFaturamento($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteContractModelTabModeloDocumento($parametros)
    {
        $this->connection->beginTransaction();

        try {
            if (isset($parametros["NRS"])) {
                $retorno = $this->findService->formatarArray($parametros["NRS"]);
                $retorno = str_replace("'", "", $retorno);
            } else {
                $retorno = $parametros;
            }

            $this->ContractModelFactory->deleteContractModelTabModeloDocumento($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteContractModelTabPerfisAcesso($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->findService->formatarArray($parametros["NRS"]);
            $this->ContractModelFactory->deleteContractModelTabPerfisAcesso($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteContractModelTabSignatariosPadroes($parametros)
    {
        $this->connection->beginTransaction();

        try {
            if (isset($parametros["NRSIGNATPADRCONT"])) {
                $retorno = $this->findService->formatarArray($parametros["NRSIGNATPADRCONT"]);
            } else {
                $retorno = $parametros;
            }
            
            $this->ContractModelFactory->deleteContractModelTabSignatariosPadroes($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getEscopoDeFaturamento($params)
    {
        // debugApi([$params]);
        $params = array(
            "NRSEQMODCONTRA"   => $params['NRSEQMODCONTRA']
        );
        return $this->ContractModelFactory->getEscopoDeFaturamento($params);
    }

    public function saveModalAddContractModelTabModeloDocumento($params)
    {
        $this->connection->beginTransaction();
        try {
            $CDOPERADOR = $this->Environment->getCdOperador();
            $NRORG = $this->Environment->getNrOrg();

            foreach ($params["IDSITUACAOCONTRATO"] as $IDSITUACAOCONTRATO) {
                $NRSEQMODELOCCONTR = $this->newCode->getCodeByTableName("FCT_MODELODOCCONTR", $NRORG, 1, 10, true);
                $this->ContractModelFactory->saveModalAddContractModelTabModeloDocumento(array(
                    "NRSEQMODELOCCONTR"          => $NRSEQMODELOCCONTR,
                    "NRSEQMODCONTRA"             => $params["NRSEQMODCONTRA"],
                    "NRSEQMODDOCFCT"             => $params["NRSEQMODDOCFCT"],
                    "IDSITUACAOCONTRATO"         => $IDSITUACAOCONTRATO,
                    "IDPERMITEDOCX"              => $params["IDPERMITEDOCX"],
                    "IDATIVO"                    => "S",
                    "NRORG"                      => $NRORG,
                    "CDOPERINCLUSAO"             => $CDOPERADOR,
                    "CDOPERULTATU"               => $CDOPERADOR,
                ));
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveModalAddContractModelTabPerfisAcesso($params)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $NRRELMODCONTPERFACESS = $this->newCode->getCodeByTableName("FCT_RELMODCONTPERFACESS", $NRORG, 1, 10, true);

            $this->ContractModelFactory->saveModalAddContractModelTabPerfisAcesso(array(
                "NRSEQMODCONTRA"             => $params["NRSEQMODCONTRA"],
                "IDATIVO"                    => "S",
                "NRPERFILACESSO"             => $params["NRPERFILACESSO"],
                "NRORG"                      => $NRORG,
                "CDOPERINCLUSAO"             => $this->Environment->getCdOperador(),
                "NRRELMODCONTPERFACESS"      => $NRRELMODCONTPERFACESS,
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveModalAddContractModelTabSignatariosPadroes($params)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $NRSIGNATPADRCONT = $this->newCode->getCodeByTableName("FCT_SIGNATPADRMODCONT", $NRORG, 1, 10, true);
            $DSEMAILNOTSIGNENV = $this->findService->formatarArray($params["DSEMAILNOTSIGNENV"]);
            $DSEMAILNOTSIGNENV = str_replace("'", "", $DSEMAILNOTSIGNENV);
            $DSEMAILNOTSIGNENV = str_replace(",", "_", $DSEMAILNOTSIGNENV);

            $this->ContractModelFactory->saveModalAddContractModelTabSignatariosPadroes(array(
                "NRSIGNATPADRCONT"           => $NRSIGNATPADRCONT,
                "NRSEQMODCONTRA"             => $params["NRSEQMODCONTRA"],
                "IDTPSIGNATARIO"             => $params["IDTPSIGNATARIO"],
                "NRPARCNEGOCIO"              => $params["NRPARCNEGOCIO"],
                "IDTIPOASSINATURA"           => $params["IDTIPOASSINATURA"],
                "IDTESTEMUNHA"               => $params["IDTESTEMUNHA"],
                "NRORDENACAO"                => $params["NRORDENACAO"],
                "DSEMAILNOTSIGNENV"          => $DSEMAILNOTSIGNENV,
                "DSEMAILNOTIFOUTROS"         => $params["DSEMAILNOTIFOUTROS"],
                "NRORG"                      => $NRORG,
                "CDOPERINCLUSAO"             => $this->Environment->getCdOperador(),
                "IDATIVO"                    => "S"
            ));

            foreach ($params["CDOPERADOR"] as $CDOPERADOR) {
                $NRRELOPERNOTIFASSELE = $this->newCode->getCodeByTableName("FCT_RELACOPERNOTIFASSELE", $NRORG, 1, 10, true);
                $this->ContractModelFactory->saveOperadoresSigPadrao(array(
                    "NRRELOPERNOTIFASSELE"   => $NRRELOPERNOTIFASSELE,
                    "NRSIGNATPADRCONT"       => $NRSIGNATPADRCONT,
                    "CDOPERADOR"             => $CDOPERADOR,
                    "NRORG"                  => $NRORG,
                    "CDOPERINCLUSAO"         => $this->Environment->getCdOperador(),
                    "IDATIVO"                => "S"
                ));
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getModeloDeDocumento()
    {
        return $this->ContractModelFactory->getModeloDeDocumento();
    }

    public function getModeloDeDocumentoAdd($params)
    {
        // debugApi([$params]);
        $params = array(
            "NRSEQMODCONTRA"   => $params['NRSEQMODCONTRA']
        );
        return $this->ContractModelFactory->getModeloDeDocumentoAdd($params);        
    }

    public function getPerfisDeAcesso()
    {
        return $this->ContractModelFactory->getPerfisDeAcesso();
    }

    public function getSignatariosContractModel()
    {
        return $this->ContractModelFactory->getSignatariosContractModel();
    }

    public function getOperadoresContractModel()
    {
        return $this->ContractModelFactory->getOperadoresContractModel();
    }

    public function getCpfSignatario($params)
    {
        $params = array(
            "NRPARCNEGOCIO"   => $params['signatario']
        );
        return $this->ContractModelFactory->getCpfSignatario($params);
    }

    public function saveModalContractModelTabEscopoFaturamento($params)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $NRESCOPOMORCONT = $this->newCode->getCodeByTableName("FCT_RELACESCOPOMODCONT", $NRORG, 1, 10, true);

            if ($params["NRESCOPOMORCONT"] == "NOVO_REGISTRO") {

                $this->ContractModelFactory->insertContractModelTabEscopoFaturamento(array(
                    "NRESCOPOMORCONT"           => $NRESCOPOMORCONT,
                    "NRSEQMODCONTRA"            => $params["NRSEQMODCONTRA"],
                    "NRTIPOORCAMENTO"           => $params["NRTIPOORCAMENTO"],
                    "IDGERAPARCAUTO"            => $params["IDGERAPARCAUTO"],
                    "IDENVEMAILPOSAPROV"        => $params["IDENVEMAILPOSAPROV"],
                    "DSEMAILENVDOCPOSAPROV"     => $params["DSEMAILENVDOCPOSAPROV"],
                    "IDATIVO"                   => "S",
                    "NRORG"                     => $NRORG,
                    "CDOPERINCLUSAO"            => $this->Environment->getCdOperador()
                ));

                if (count($params["NRTIPOANEXO"]) > 0) {
                    
                    foreach ($params["NRTIPOANEXO"] as $anexo) {
                        $NRRELACESCOPOANEXOMODCONT = $this->newCode->getCodeByTableName("FCT_RELACESCOPOANEXOMODCONT", $NRORG, 1, 10, true);

                        $this->ContractModelFactory->insertAnexoEscFatModCont(array(
                            "NRRELACESCOPOANEXOMODCONT"     => $NRRELACESCOPOANEXOMODCONT,
                            "NRESCOPOMORCONT"               => $NRESCOPOMORCONT,
                            "NRTIPOANEXO"                   => $anexo["NRTIPOANEXO"],
                            "NRORG"                         => $NRORG,
                            "IDATIVO"                       => "S",
                            "CDOPERINCLUSAO"                => $this->Environment->getCdOperador(),
                            "NRORGINCLUSAO"                 => $NRORG,
                        ));
                    }
                }

            } else {
                $this->ContractModelFactory->updateContractModelTabEscopoFaturamento(array(
                    "NRESCOPOMORCONT"           => $params["NRESCOPOMORCONT"],
                    "NRSEQMODCONTRA"            => $params["NRSEQMODCONTRA"],
                    "NRTIPOORCAMENTO"           => $params["NRTIPOORCAMENTO"],
                    "IDGERAPARCAUTO"            => $params["IDGERAPARCAUTO"],
                    "IDENVEMAILPOSAPROV"        => $params["IDENVEMAILPOSAPROV"],
                    "DSEMAILENVDOCPOSAPROV"     => $params["DSEMAILENVDOCPOSAPROV"],
                    "IDATIVO"                   => "S",
                    "NRORG"                     => $NRORG,
                    "CDOPERULTATU"              => $this->Environment->getCdOperador(),
                    "NRORGULTATU"               => $NRORG,
                ));

                $modificado = $params["NRTIPOANEXO"];
                $original = $params["NRTIPOANEXO_BACKUP"];

                $removidos = array_filter($original, function ($itemOriginal) use ($modificado) {
                    foreach ($modificado as $itemModificado) {
                        if ($itemOriginal['NRTIPOANEXO'] === $itemModificado['NRTIPOANEXO']) {
                            return false; // Se o ID está presente, não foi removido
                        }
                    }
                    return true; // Se não encontrou o ID no array modificado, foi removido
                });

                // Função para verificar os itens adicionados
                $adicionados = array_filter($modificado, function ($itemModificado) use ($original) {
                    foreach ($original as $itemOriginal) {
                        if ($itemModificado['NRTIPOANEXO'] === $itemOriginal['NRTIPOANEXO']) {
                            return false; // Se o ID está presente no original, não foi adicionado
                        }
                    }
                    return true; // Se não encontrou o ID no array original, foi adicionado
                });

                if (count($adicionados) > 0) {
                    
                    foreach ($adicionados as $adicionado) {
                        $NRRELACESCOPOANEXOMODCONT = $this->newCode->getCodeByTableName("FCT_RELACESCOPOANEXOMODCONT", $NRORG, 1, 10, true);

                        $this->ContractModelFactory->insertAnexoEscFatModCont(array(
                            "NRRELACESCOPOANEXOMODCONT"     => $NRRELACESCOPOANEXOMODCONT,
                            "NRESCOPOMORCONT"               => $params["NRESCOPOMORCONT"],
                            "NRTIPOANEXO"                   => $adicionado["NRTIPOANEXO"],
                            "NRORG"                         => $NRORG,
                            "IDATIVO"                       => "S",
                            "CDOPERINCLUSAO"                => $this->Environment->getCdOperador(),
                            "NRORGINCLUSAO"                 => $NRORG,
                        ));
                    }
                }

                if (count($removidos) > 0) {
                    foreach ($removidos as $removido) {
                        if (isset($removido['NRTIPOANEXO'])) {
                            $this->ContractModelFactory->deleteAnexoEscFatModCont(array(
                                "NRESCOPOMORCONT"   => $params["NRESCOPOMORCONT"],
                                "NRTIPOANEXO"       => $removido["NRTIPOANEXO"],
                            ));
                        }                       
                    }
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function editContractModelTabModeloDocumento($params)
    {
        $this->connection->beginTransaction();
       
        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();
            foreach ($params["EDITEDROWS"] as $key => $row) {
                $this->deleteContractModelTabModeloDocumento($row['NRSEQMODELOCCONTR']);
                $this->saveModalAddContractModelTabModeloDocumento($row);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function editContractModelTabSignatariosPadroes($params)
    {
        $this->connection->beginTransaction();
       
        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();

            $this->deleteContractModelTabSignatariosPadroes($params['NRSIGNATPADRCONT']);
            $this->saveModalAddContractModelTabSignatariosPadroes($params);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function editContractModel($params)
    {

        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();
 
            $this->ContractModelFactory->editContractModel(array(
                "NRSEQMODCONTRA"             => $params["NRSEQMODCONTRA"],
                "NMMODCONTRATO"              => $params["NMMODCONTRATO"],
                "IDATIVO"                    => $params["IDATIVO"],
                "CDMODCONTRATO"              => $params["CDMODCONTRATO"],
                "IDGERAPROPOSTA"             => $params["IDGERAPROPOSTA"],
                "CDCENTCUST"                 => $params["CDCENTCUST"],
                "IDGERACONTFOR"              => $params["IDGERACONTFOR"],
                "NRDIASVIGCONT"              => $params["NRDIASVIGCONT"],
                "IDRENOVACAOAUTOMATICA"      => $params["IDRENOVACAOAUTOMATICA"],
                "NRDIASPRORGCONT"            => $params["NRDIASPRORGCONT"],
                "IDBLOQALTDTINICONTRENEG"    => $params["IDBLOQALTDTINICONTRENEG"],
                "IDREAJUSTEAUTOMATICO"       => $params["IDREAJUSTEAUTOMATICO"],
                "NRDIASREAJAUTOMATICO"       => $params["NRDIASREAJAUTOMATICO"],
                "IDINTEGRACAOFM"             => $params["IDINTEGRACAOFM"],
                "IDINTEGRALOCAISFM"          => $params["IDINTEGRALOCAISFM"],
                "IDINTEGRAMEDICAOFM"         => $params["IDINTEGRAMEDICAOFM"],
                "IDINTEGRACAOHCM"            => $params["IDINTEGRACAOHCM"],
                "TPESTRUTINTEGRAHCM"         => $params["TPESTRUTINTEGRAHCM"],
                "IDINTEGRAESCALAHCM"         => $params["IDINTEGRAESCALAHCM"],
                "IDAGRUPAESCALAHCM"          => $params["IDAGRUPAESCALAHCM"],
                "IDASSELETRDIGITAL"          => $params["IDASSELETRDIGITAL"],
                "NRTIPODOCCERTSIGN"          => $params["NRTIPODOCCERTSIGN"],
                "IDENVIADOCOPERRESPCONCASS"  => $params["IDENVIADOCOPERRESPCONCASS"],
                "NRORG"                      => $nrorg,
                "CDOPERULTATU"               => $operador,
                "IDINTEGRACAOTECFOOD"        => $params["IDINTEGRACAOTECFOOD"],
                "IDTIPOSERVINTTECFOOD"       => $params["IDTIPOSERVINTTECFOOD"],
            ));

            $this->insereEditaServicosExecHCM($params["NRSEQMODCONTRA"], $params["CDSERVORCAEXECHCM"], $nrorg, $operador);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function insereEditaServicosExecHCM($nrSeqModContra, $servicosExecHCM, $nrorg, $operador) {

        if ($servicosExecHCM == null){

            $params = array(
                "NRSEQMODCONTRA" => $nrSeqModContra,
                "NRORG"          => $nrorg
            );
            
            $this->ContractModelFactory->excluiServicosExecHCMTotal($params);

        }
        else { 

            /* 1 - Exclui todos que não estao marcados */
            $servExecHCM =  implode(",", $servicosExecHCM);

            $params = array(
                "NRSEQMODCONTRA" => $nrSeqModContra,
                "SERVICOS"       => $servExecHCM,
                "NRORG"          => $nrorg
            );
            
            $this->ContractModelFactory->excluiServicosExecHCMEspecifico($params);

            /* 2 - insere os que não estão inseridos*/

            foreach ($servicosExecHCM as $srvExecHCM) {
                
                $params = array(
                    "NRSEQMODCONTRA"    => $nrSeqModContra,
                    "CDSERVORCAEXECHCM" => $srvExecHCM,
                    "NRORG"             => $nrorg
                );

                $valida = $this->ContractModelFactory->validaExisteServicosExecHCM($params);

                if ($valida == 0){

                    $nrServEexecaoHCM = $this->newCode->getCodeByTableName("FCT_SERVEXCECAOHCM", $nrorg, 1, 10, true);

                    $params = array(
                        "NRSERVEXCECAOHCM"  => $nrServEexecaoHCM,
                        "NRSEQMODCONTRA"    => $nrSeqModContra,
                        "CDSERVORCAEXECHCM" => $srvExecHCM,                    
                        "NRORG"             => $nrorg,
                        "OPERADOR"          => $operador
                    );

                    $this->ContractModelFactory->insereServicosExecHCM($params);
                }

            }

        }

    }
    
}
